@@ -145,9 +145,9 @@ static void cc_miller(element_t res, mpz_t q, point_t P,
 	//c = -(Z.y + a * Z.x);
 	//but we multiply by 2*Z.y to avoid division
 
-	//a = -Zx * (Zx + Zx + Zx + twicea_2) - a_4;
+	//a = -Zx * (3 Zx + twicea_2) - a_4;
 	//Common curves: a2 = 0 (and cc->a is a_4), so
-	//a = -(Zx (Zx + Zx + Zx) + cc->a)
+	//a = -(3 Zx^2 + cc->a)
 	//b = 2 * Zy
 	//c = -(2 Zy^2 + a Zx);
 	element_ptr Zx = Z->x;
@@ -163,9 +163,8 @@ static void cc_miller(element_t res, mpz_t q, point_t P,
 	    return;
 	}
 
-	element_add(a, Zx, Zx);
-	element_add(a, a, Zx);
-	element_mul(a, a, Zx);
+	element_square(a, Zx);
+	element_mul_si(a, a, 3);
 	element_add(a, a, cc->a);
 	element_neg(a, a);
 
@@ -249,8 +248,8 @@ static void cc_miller(element_t res, mpz_t q, point_t P,
     m = mpz_sizeinbase(q, 2) - 2;
 
     while(m >= 0) {
-	element_mul(v, v, v);
-	element_mul(vd, vd, vd);
+	element_square(v, v);
+	element_square(vd, vd);
 	do_tangent(v);
 	point_double(Z, Z);
 	do_vertical(vd);
@@ -430,7 +429,7 @@ static void cc_miller_no_denom(element_t res, mpz_t q, point_t P,
 
 	//a = -Zx * (Zx + Zx + Zx + twicea_2) - a_4;
 	//Common curves: a2 = 0 (and cc->a is a_4), so
-	//a = -(Zx (Zx + Zx + Zx) + cc->a)
+	//a = -(3 Zx^2 + cc->a)
 	//b = 2 * Zy
 	//c = -(2 Zy^2 + a Zx);
 	element_ptr Zx = Z->x;
@@ -446,9 +445,8 @@ static void cc_miller_no_denom(element_t res, mpz_t q, point_t P,
 	    return;
 	}
 
-	element_add(a, Zx, Zx);
-	element_add(a, a, Zx);
-	element_mul(a, a, Zx);
+	element_square(a, Zx);
+	element_mul_si(a, a, 3);
 	element_add(a, a, cc->a);
 	element_neg(a, a);
 
@@ -530,7 +528,7 @@ static void cc_miller_no_denom(element_t res, mpz_t q, point_t P,
     m = mpz_sizeinbase(q, 2) - 2;
 
     while(m >= 0) {
-	element_mul(v, v, v);
+	element_square(v, v);
 	do_tangent();
 	point_double(Z, Z);
 	if (mpz_tstbit(q, m)) {
@@ -573,7 +571,7 @@ static void cc_tatepower_even_k(element_ptr out, element_ptr in, pairing_t pairi
 	    element_add(e0re, e0re, e2);
 	    polymod_const_mul(e2, inim[1], e);
 	    element_add(e0im, e0im, e2);
-	    element_mul(e1, e, e);
+	    element_square(e1, e);
 	    polymod_const_mul(e2, inre[2], e1);
 	    element_add(e0re, e0re, e2);
 	    polymod_const_mul(e2, inim[2], e1);
@@ -748,8 +746,7 @@ static void pairing_init_c_param_even_k(pairing_t pairing, c_param_t param)
     element_init(p->nqrinv, p->Fqd);
     element_invert(p->nqrinv, field_get_nqr(p->Fqd));
     element_init(p->nqrinv2, p->Fqd);
-    //TODO: element_square
-    element_mul(p->nqrinv2, p->nqrinv, p->nqrinv);
+    element_square(p->nqrinv2, p->nqrinv);
 
     pairing->G1 = malloc(sizeof(field_t));
     pairing->G2 = malloc(sizeof(field_t));
