@@ -189,7 +189,7 @@ static void cc_mul(point_ptr r, mpz_ptr n, point_ptr p)
     point_init(result, r->curve);
     point_set_inf(result);
 
-    if (mpz_sgn(n)) for (s = mpz_sizeinbase(n, 2) - 1; s>=0; s--) {
+    if (!mpz_is0(n)) for (s = mpz_sizeinbase(n, 2) - 1; s>=0; s--) {
 	point_double(result, result);
 	if (mpz_tstbit(n, s)) {
 	    point_add(result, result, p);
@@ -422,6 +422,7 @@ void field_clear_curve_group(field_t f)
 void field_init_curve_group(field_t f, curve_t c, mpz_t cofac)
 {
     curve_group_data_ptr p;
+    field_init(f);
     p = f->data = malloc(sizeof(curve_group_data_t));
     p->curve = c;
     mpz_init(p->cofac);
