@@ -14,8 +14,8 @@
 struct field_s;
 
 struct element_s {
-    struct field_s *field;
     void *data;
+    struct field_s *field;
 };
 typedef struct element_s *element_ptr;
 typedef struct element_s element_t[1];
@@ -36,6 +36,7 @@ struct field_s {
     void (*mul_mpz)(element_ptr, element_ptr, mpz_ptr);
     void (*mul_si)(element_ptr, element_ptr, signed long int);
     void (*square)(element_ptr, element_ptr);
+    void (*pow)(element_ptr, element_ptr, mpz_ptr);
     void (*invert)(element_ptr, element_ptr);
     void (*neg)(element_ptr, element_ptr);
     int (*cmp)(element_ptr, element_ptr);
@@ -52,6 +53,7 @@ struct field_s {
     mpz_t order; //-1 for infinite order
     element_ptr nqr; //nonquadratic residue
     void *data;
+    void (*to_mpz)(mpz_ptr, element_ptr); //does this make sense for all fields?
 };
 typedef struct field_s *field_ptr;
 typedef struct field_s field_t[1];
@@ -130,6 +132,11 @@ static inline void element_square(element_ptr n, element_ptr a)
     n->field->square(n, a);
 }
 
+static inline void element_pow(element_ptr x, element_ptr a, mpz_ptr n)
+{
+    x->field->pow(x, a, n);
+}
+
 static inline void element_neg(element_ptr n, element_ptr a)
 {
     n->field->neg(n, a);
@@ -185,6 +192,11 @@ static inline int element_from_bytes(element_ptr a, unsigned char *data)
     return a->field->from_bytes(a, data);
 }
 
+static inline void element_to_mpz(mpz_ptr z, element_ptr a)
+{
+    a->field->to_mpz(z, a);
+}
+
 static inline int element_length_in_bytes(element_ptr a)
 {
     if (a->field->fixed_length_in_bytes < 0) {
@@ -195,7 +207,7 @@ static inline int element_length_in_bytes(element_ptr a)
 }
 
 void field_init_fp(field_ptr f, mpz_t prime);
-void element_pow(element_ptr x, element_ptr a, mpz_ptr n);
+
 void element_pow2(element_ptr x, element_ptr a1, mpz_ptr n1,
                                  element_ptr a2, mpz_ptr n2);
 void element_pow3(element_ptr x, element_ptr a1, mpz_ptr n1,
@@ -215,4 +227,10 @@ static inline void field_clear(field_ptr f)
 element_ptr field_get_nqr(field_ptr f);
 
 void field_init(field_ptr f);
+
+static inline int mpz_is0(mpz_ptr z)
+{
+    return !mpz_sgn(z);
+    //return !mpz_cmp_ui(z, 0);
+}
 #endif //FIELD_H
