@@ -41,14 +41,25 @@ static void zp_out_str(FILE *stream, int base, element_ptr e)
 
 static void zp_add(element_ptr n, element_ptr a, element_ptr b)
 {
+    /*
     mpz_add(n->data, a->data, b->data);
     mpz_mod(n->data, n->data, n->field->order);
+    */
+    //This seems faster:
+    mpz_add(n->data, a->data, b->data);
+    if (mpz_cmp(n->data, n->field->order) >= 0) {
+	mpz_sub(n->data, n->data, n->field->order);
+    }
 }
 
 static void zp_sub(element_ptr n, element_ptr a, element_ptr b)
 {
+    //mpz_sub(n->data, a->data, b->data);
+    //mpz_mod(n->data, n->data, n->field->order);
     mpz_sub(n->data, a->data, b->data);
-    mpz_mod(n->data, n->data, n->field->order);
+    if (mpz_sgn((mpz_ptr) n->data) < 0) {
+	mpz_add(n->data, n->data, n->field->order);
+    }
 }
 
 static void zp_mul(element_ptr n, element_ptr a, element_ptr b)
@@ -69,6 +80,11 @@ static void zp_mul_si(element_ptr n, element_ptr a, signed long int z)
     mpz_mod(n->data, n->data, n->field->order);
 }
 
+static void zp_pow(element_ptr n, element_ptr a, mpz_ptr z)
+{
+    mpz_powm(n->data, a->data, z, n->field->order);
+}
+
 static void zp_set(element_ptr n, element_ptr a)
 {
     mpz_set(n->data, a->data);
@@ -76,7 +92,11 @@ static void zp_set(element_ptr n, element_ptr a)
 
 static void zp_neg(element_ptr n, element_ptr a)
 {
-    if (mpz_sgn((mpz_ptr) a->data)) mpz_sub(n->data, n->field->order, a->data);
+    if (mpz_is0(a->data)) {
+	mpz_set_ui(n->data, 0);
+    } else {
+	mpz_sub(n->data, n->field->order, a->data);
+    }
 }
 
 static void zp_invert(element_ptr n, element_ptr a)
@@ -107,7 +127,7 @@ static int zp_is1(element_ptr n)
 
 static int zp_is0(element_ptr n)
 {
-    return !mpz_cmp_ui((mpz_ptr) n->data, 0);
+    return mpz_is0(n->data);
 }
 
 static int zp_cmp(element_ptr a, element_ptr b)
@@ -118,7 +138,7 @@ static int zp_cmp(element_ptr a, element_ptr b)
 static int zp_is_sqr(element_ptr a)
 {
     //0 is a square
-    if (!mpz_sgn((mpz_ptr) a->data)) return 1;
+    if (mpz_is0(a->data)) return 1;
     return mpz_legendre(a->data, a->field->order) == 1;
 }
 
@@ -215,6 +235,11 @@ static int zp_from_bytes(element_t e, unsigned char *data)
     return n;
 }
 
+static void zp_to_mpz(mpz_ptr z, element_ptr a)
+{
+    mpz_set(z, a->data);
+}
+
 void field_init_fp(field_ptr f, mpz_t prime)
 {
     field_init(f);
@@ -229,6 +254,7 @@ void field_init_fp(field_ptr f, mpz_t prime)
     f->mul = zp_mul;
     f->mul_mpz = zp_mul_mpz;
     f->mul_si = zp_mul_si;
+    f->pow = zp_pow;
     f->neg = zp_neg;
     f->cmp = zp_cmp;
     f->invert = zp_invert;
@@ -243,6 +269,7 @@ void field_init_fp(field_ptr f, mpz_t prime)
     f->field_clear = zp_field_clear;
     f->to_bytes = zp_to_bytes;
     f->from_bytes = zp_from_bytes;
+    f->to_mpz = zp_to_mpz;
 
     mpz_set(f->order, prime);
     f->data = NULL;
